local _G = GLOBAL

if _G.TheNet and _G.TheNet:GetIsServer() then
	return
end

---------------Load Cofig
local function GetConfig(s,default)
	local c=GetModConfigData(s)
	if c==nil then
		c=default
	end
	if type(c)=="table" then
		c=c.option_data
	end
	return c
end
local Save_Connect_Data = GetConfig("SAVE_CONNECT_DATA", true)

-- URLs
local GETIP_VISIT = "http://ipecho.net"
-- local GETIP_URL = "http://ipecho.net/plain"
local GETIP_URL = "https://api.ipify.org/"
local SERVERLIST_URL = "https://my.jacklul.com/dstservers/"

local steam_support_languages = {
	schinese = "chs",
	tchinese = "cht",
	russian = "ru",
}

local support_languages = {
	chs = true,
	cht = true,
	cn = "chs",
	zh_CN = "chs",
	TW = "cht",
	ru = true,
}

-- Local
local require = _G.require
local VisitURL = _G.VisitURL
local TheNet = _G.TheNet
local TheSim = _G.TheSim
local tonumber = _G.tonumber
local tostring = _G.tostring
STRINGS = _G.STRINGS

-- Path
local fileip = "mod_config_data/connect_ip_save"
local fileport = "mod_config_data/connect_port_save"

-- Menu Position
local Xpos = _G.RESOLUTION_X/2 -180
local Ypos = -220

-- Require
local InputDialogScreen = require "screens/inputdialog"
local PopupDialogScreen = require "screens/popupdialog"
local Text = require "widgets/text"
local Button = require "widgets/button"
local Image = require "widgets/image"
local Menu = require "widgets/menu"

-- Strings
--modimport("Strings/default.lua")
local steamlang = _G.TheNet:GetLanguageCode() or nil
if steamlang and steam_support_languages[steamlang] then
	print("Get your language from steam!")
	modimport("Strings/connect_"..steam_support_languages[steamlang]..".lua")
else
	local lang = _G.LanguageTranslator.defaultlang or nil
	if lang ~= nil and support_languages[lang] ~= nil then
		if support_languages[lang] ~= true then
			lang = support_languages[lang]
		end
		print("Get your language from language mod!")
		modimport("Strings/connect_"..lang..".lua")
	else
		modimport("Strings/connect.lua")
	end
end

-- Store Server
local server={}
	server.ip = nil
	server.port = 10999
	server.guid = 1234567898765432123

-- Function Data Manage
local function Connect_Data_Set(filename, str)
	TheSim:SetPersistentString(filename, str, false,
		function()
			print ("[OK]Connect data("..str..") saved! Path: "..filename)
		end
	)
end

local function Connect_Data_Del(filename)
	TheSim:ErasePersistentString(filename,
		function()
			print ("[OK]Delete Connect data! Path: "..filename)
		end
	)
end

if Save_Connect_Data then
	TheSim:GetPersistentString(fileip,
		function(load_success, str)
			if load_success and str ~= nil and #str > 0 then
				server.ip = str
			else
				server.ip = ""
			end
		end
	)
	TheSim:GetPersistentString(fileport,
		function(load_success, str)
			if load_success and str ~= nil and #str > 0 then
				server.port = tonumber(str)
			else
				server.port = 10999
			end
		end
	)
	print ("Loading Connect Data !\n"..server.ip.."\n"..tostring(server.port).."\n")
end

-- Function connect
local function Join_Server()

	if server.ip == nil or server.ip == "" or server.ip == " " then
		return
	end

	if Save_Connect_Data then
		Connect_Data_Set(fileip, server.ip)
		Connect_Data_Set(fileport, tostring(server.port))
	end

	ShowConnectmenu()

	local start_worked = TheNet:StartClient( server.ip, server.port, server.guid )
	if start_worked then
		local consolelog = "Connecting "..server.ip..":"..tostring(server.port)
		print(consolelog)
	end

end

-- Thank for this function from stackoverflow.com!
-- Lua function check if ipv4 or ipv6 or string
function IsRightIP( str )
	local R = {ERROR = 0, IPV4 = 1, IPV6 = 2, STRING = 3}
	if type(str) ~= "string" then return R.ERROR end

	-- check for format 1.11.111.111 for ipv4
	local chunks = {str:match("(%d+)%.(%d+)%.(%d+)%.(%d+)")}
	if #chunks == 4 then
		for _,v in pairs(chunks) do
			if tonumber(v) > 255 then return R.STRING end
		end
		return R.IPV4
	end

	-- check for ipv6 format, should be 8 'chunks' of numbers/letters
	-- without trailing chars
	local chunks = {str:match(("([a-fA-F0-9]*):"):rep(8):gsub(":$","$"))}
	if #chunks == 8 then
		for _,v in pairs(chunks) do
			if #v > 0 and tonumber(v, 16) > 65535 then return R.STRING end
		end
		return R.IPV6
	end

	return R.STRING
end

-- Simple Port Check
function IsRightPort( str )
    return string.match(str,"^%d+$") == str
end

-- ConnectButton Called
local function OnConnectButton()

	local function port_get()
		local port_prompt_screen

		port_prompt_screen = InputDialogScreen( STRINGS.CONNECTSTRINGS.TITLE.PORT,
		{
			{
				text = STRINGS.UI.SERVERLISTINGSCREEN.CANCEL,
				cb = function()
					TheFrontEnd:PopScreen()
				end
			},
			{
				text = STRINGS.CONNECTSTRINGS.BUTTON.RESET,
				cb = function()
					port_prompt_screen.edit_text:SetString("")
				end
			},
			{
				text = STRINGS.UI.SERVERLISTINGSCREEN.OK,
				cb = function()
					TheFrontEnd:PopScreen()
					if not IsRightPort(port_prompt_screen:GetActualString()) then
						if port_prompt_screen:GetActualString() == "" or port_prompt_screen:GetActualString() == " " then
							server.port = 10999
							Join_Server()
						else
							TheFrontEnd:PushScreen(PopupDialogScreen(
								STRINGS.CONNECTSTRINGS.TITLE.WRONGPORT,
								STRINGS.CONNECTSTRINGS.BODY.WRONGPORT,
								{{
									text = STRINGS.UI.SERVERLISTINGSCREEN.OK,
									cb = function() TheFrontEnd:PopScreen() port_get() end
								}}
							))
						end
					else
						server.port = tonumber(port_prompt_screen:GetActualString())
						Join_Server()
					end
				end
			},
		},
		true )

		local VALID_CHARS = [[0123456789]]
		port_prompt_screen:SetValidChars(VALID_CHARS)

		port_prompt_screen.edit_text.OnTextEntered = function()
			TheFrontEnd:PopScreen()
			if not IsRightPort(port_prompt_screen:GetActualString()) then
				if port_prompt_screen:GetActualString() == "" or port_prompt_screen:GetActualString() == " " then
					server.port = 10999
					Join_Server()
				else
					TheFrontEnd:PushScreen(PopupDialogScreen(
						STRINGS.CONNECTSTRINGS.TITLE.WRONGPORT,
						STRINGS.CONNECTSTRINGS.BODY.WRONGPORT,
						{{
							text = STRINGS.UI.SERVERLISTINGSCREEN.OK,
							cb = function() TheFrontEnd:PopScreen() port_get() end
						}}
					))
				end
			else
				server.port = tonumber(port_prompt_screen:GetActualString())
				Join_Server()
			end
		end

		TheFrontEnd:PushScreen(port_prompt_screen)
		port_prompt_screen.edit_text:SetForceEdit(true)
		--port_prompt_screen.edit_text:SetAllowClipboardPaste(true)
		port_prompt_screen.edit_text:SetEditing(true)
		port_prompt_screen.edit_text:OnControl(CONTROL_ACCEPT, false)
	end

	local function ip_get()
		local ip_prompt_screen

		ip_prompt_screen = InputDialogScreen( STRINGS.CONNECTSTRINGS.TITLE.IP,
		{
			{
				text = STRINGS.UI.SERVERLISTINGSCREEN.CANCEL,
				cb = function()
					TheFrontEnd:PopScreen()
				end
			},
			{
				text = STRINGS.CONNECTSTRINGS.BUTTON.RESET,
				cb = function()
					ip_prompt_screen.edit_text:SetString("")
				end
			},
			{
				text = STRINGS.UI.SERVERLISTINGSCREEN.OK,
				cb = function()
					TheFrontEnd:PopScreen()
					if IsRightIP(ip_prompt_screen:GetActualString()) == 0 or IsRightIP(ip_prompt_screen:GetActualString()) == 3 then
						TheFrontEnd:PushScreen(PopupDialogScreen(
							STRINGS.CONNECTSTRINGS.TITLE.WRONGIP,
							STRINGS.CONNECTSTRINGS.BODY.WRONGIP,
							{{
								text = STRINGS.UI.SERVERLISTINGSCREEN.OK,
								cb = function() TheFrontEnd:PopScreen() ip_get() end
							}}
						))
					else
						server.ip = ip_prompt_screen:GetActualString()
						port_get()
					end
				end
			},
		},
		true )

		local VALID_CHARS = [[abcdefABCDEF0123456789.:"]]
		ip_prompt_screen:SetValidChars(VALID_CHARS)

		ip_prompt_screen.edit_text.OnTextEntered = function()
			TheFrontEnd:PopScreen()
			if IsRightIP(ip_prompt_screen:GetActualString()) == 0 or IsRightIP(ip_prompt_screen:GetActualString()) == 3 then
				TheFrontEnd:PushScreen(PopupDialogScreen(
					STRINGS.CONNECTSTRINGS.TITLE.WRONGIP,
					STRINGS.CONNECTSTRINGS.BODY.WRONGIP,
					{{
						text = STRINGS.UI.SERVERLISTINGSCREEN.OK,
						cb = function() TheFrontEnd:PopScreen() ip_get() end
					}}
				))
			else
				server.ip = ip_prompt_screen:GetActualString()
				port_get()
			end
		end

		TheFrontEnd:PushScreen(ip_prompt_screen)
		ip_prompt_screen.edit_text:SetForceEdit(true)
		--ip_prompt_screen.edit_text:SetAllowClipboardPaste(true)
		ip_prompt_screen.edit_text:SetEditing(true)
		ip_prompt_screen.edit_text:OnControl(CONTROL_ACCEPT, false)
	end

	ip_get()
end

-- RetryButton Called
local function OnRetryButton()
	Join_Server()
end

-- CleanButton Called
local function OnCleanButton()
	Connect_Data_Del(fileip)
	Connect_Data_Del(fileport)
	server.ip = nil
	server.port = 10999
	HideConnectmenu()
end

-- GetIPButton Called
local function OnGetIPButton()
	TheSim:QueryServer( GETIP_URL, function(...) OnIPQueryComplete(...) end, "GET" )
	TheFrontEnd:PushScreen(PopupDialogScreen(
		STRINGS.CONNECTSTRINGS.TITLE.WAIT,
		STRINGS.CONNECTSTRINGS.BODY.WAIT,
		{{
			text = STRINGS.UI.SERVERLISTINGSCREEN.OK,
			cb = function() end
		}}
	))
end

-- GetIPButton Fallback
function OnIPQueryComplete( result, isSuccessful, resultCode )
	TheFrontEnd:PopScreen()
	if isSuccessful and string.len(result) > 1 and resultCode == 200 then
		local ShowIP_screen

 		ShowIP_screen = InputDialogScreen( STRINGS.CONNECTSTRINGS.TITLE.SHOWIP,
		{
			{
				text = STRINGS.UI.SERVERLISTINGSCREEN.OK,
				cb = function()
					TheFrontEnd:PopScreen()
				end
			},
			{
				text = STRINGS.CONNECTSTRINGS.BUTTON.VISITIP,
				cb = function()
					TheFrontEnd:PopScreen()
					VisitURL(GETIP_VISIT, false)
				end
			},
		},
		true )

		ShowIP_screen.edit_text.OnTextEntered = function()
			TheFrontEnd:PopScreen()
		end

		TheFrontEnd:PushScreen(ShowIP_screen)
		ShowIP_screen.edit_text:SetForceEdit(false)
		--ShowIP_screen.edit_text:SetAllowClipboardPaste(false)
		ShowIP_screen.edit_text:SetEditing(false)
		ShowIP_screen.edit_text:SetString(result)
		ShowIP_screen.edit_text:OnControl(CONTROL_ACCEPT, false)
	end
end

-- ServerListButton Called
local function OnServerListButton()
	VisitURL(SERVERLIST_URL, true)
end

-- Add Functions in Multiplayermainscreen
AddClassPostConstruct("screens/multiplayermainscreen", function(self)

	function self.ConnectInit()
		local ConnectMenuitems = {}

		-- Make Button
		local function MakeConnectButton(text, onclick, tooltip)
			local btn = Button()
			btn:SetFont(_G.BUTTONFONT)
			btn:SetDisabledFont(_G.BUTTONFONT)
			btn:SetTextColour(_G.unpack(_G.GOLD))
			btn:SetTextFocusColour(1, 1, 1, 1)
			btn:SetText(text, true)
			btn.text:SetRegionSize(180,40)
			btn.text:SetHAlign(_G.ANCHOR_LEFT)
			btn.text_shadow:SetRegionSize(180,40)
			btn.text_shadow:SetHAlign(_G.ANCHOR_LEFT)
			btn:SetTextSize(35)

			btn.image = btn:AddChild(Image("images/frontscreen.xml", "highlight_hover.tex"))
			btn.image:MoveToBack()
			btn.image:SetScale(.6)
			btn.image:SetPosition(-20,3)
			btn.image:SetClickable(false)
			btn.image:Hide()

			btn.bg = btn:AddChild(Image("images/ui.xml", "blank.tex"))
			local w,h = btn.text:GetRegionSize()
			btn.bg:ScaleToSize(200, h+15)

			btn.OnGainFocus = function()
				if btn.text then btn.text:SetColour(btn.textfocuscolour[1],btn.textfocuscolour[2],btn.textfocuscolour[3],btn.textfocuscolour[4]) end
				TheFrontEnd:GetSound():PlaySound("dontstarve/HUD/click_mouseover")
				btn.image:Show()
				self.tooltip:SetString(tooltip)
				self.tooltip_shadow:SetString(tooltip)

				self.updatename_root:CancelMoveTo()
				self.updatename_root:SetPosition( self.updatename_root_off )
			end

			btn.OnLoseFocus = function()
				if btn:IsEnabled() and not btn.selected then
					btn.text:SetColour(btn.textcolour)
				end
				if btn.o_pos then
					btn:SetPosition(btn.o_pos)
				end
				btn.down = false

				btn.image:Hide()
				if not self.Connectmenu.focus then
					self.tooltip:SetString("")
					self.tooltip_shadow:SetString("")
					self.updatename_root:MoveTo( self.updatename_root_off, self.updatename_root_on, .5 )
				end
			end
			btn:SetOnClick(onclick)
			-- btn:SetScale(.75)

			return btn

		end

		local ConnectButton = MakeConnectButton(STRINGS.CONNECTSTRINGS.BUTTON.CONNECT, function() OnConnectButton() end, STRINGS.CONNECTSTRINGS.TOOLTIP.CONNECT)
		RetryButton = MakeConnectButton(STRINGS.CONNECTSTRINGS.BUTTON.RETRY, function() OnRetryButton() end, STRINGS.CONNECTSTRINGS.TOOLTIP.RETRY)
		CleanButton = MakeConnectButton(STRINGS.CONNECTSTRINGS.BUTTON.RESET, function() OnCleanButton() end, STRINGS.CONNECTSTRINGS.TOOLTIP.RESET)
		local GetIPButton = MakeConnectButton(STRINGS.CONNECTSTRINGS.BUTTON.GETIP, function() OnGetIPButton() end, STRINGS.CONNECTSTRINGS.TOOLTIP.GETIP)
		local ServerListButton = MakeConnectButton(STRINGS.CONNECTSTRINGS.BUTTON.SERVER, function() OnServerListButton() end, STRINGS.CONNECTSTRINGS.TOOLTIP.SERVER)

		ConnectMenuitems =
		{
			{widget = ConnectButton},
			{widget = GetIPButton},
			{widget = ServerListButton},
			{widget = RetryButton},
			{widget = CleanButton},
		}

		self.Connectmenu = self.fixed_root:AddChild(Menu(ConnectMenuitems, 43, nil, nil, true))
		self.Connectmenu:SetPosition(Xpos, Ypos)

		if server.ip ~= nil and server.ip ~= "" and server.ip ~= " " then
			ShowConnectmenu()
		else
			HideConnectmenu()
		end

	end

	function ShowConnectmenu()
		CleanButton:Show()
		RetryButton:Show()
	end

	function HideConnectmenu()
		CleanButton:Hide()
		RetryButton:Hide()
	end

	self.ConnectInit()

end)
